#!/bin/bash

echo "MySQL: inserting test data (root/signer/vault/scep certificates)"

#
# Files for /etc
#
BASE="/etc/openxpki/ssl/ca-one/";
if [ ! -d "$BASE" ]; then mkdir -p "$BASE"; fi

if [ -e "$BASE/ca-root-1.crt" ] ||  [ -e "$BASE/ca-root-1.pem" ]; then
   echo "found exisiting ca files in directory, please remove all files!"
   exit 1
fi

cp $(dirname $0)/certificates/* $BASE/

#
# Database entries
#
DBPASS=""
test ! -z "$OXI_TEST_DB_MYSQL_DBPASSWORD" && DBPASS="-p$OXI_TEST_DB_MYSQL_DBPASSWORD" || true

cat <<'__SQL' | mysql -h $OXI_TEST_DB_MYSQL_DBHOST -P $OXI_TEST_DB_MYSQL_DBPORT -u$OXI_TEST_DB_MYSQL_DBUSER $DBPASS $OXI_TEST_DB_MYSQL_NAME

INSERT INTO `aliases` (`identifier`,`pki_realm`,`alias`,`group_id`,`generation`,`notafter`,`notbefore`) VALUES ('NZA-k3-yaMBQdsiAkbcfAeo8vQo','ca-one','ca-one-scep-1','ca-one-scep',1,4294967295,1485469746);
INSERT INTO `aliases` (`identifier`,`pki_realm`,`alias`,`group_id`,`generation`,`notafter`,`notbefore`) VALUES ('mIqp8XGxjquD0MtxsiFhv7JFfGk','ca-one','ca-one-signer-1','ca-one-signer',1,4294967295,1485469746);
INSERT INTO `aliases` (`identifier`,`pki_realm`,`alias`,`group_id`,`generation`,`notafter`,`notbefore`) VALUES ('IPkm4dSkDn1oPuWIRZjIxvvpsno','ca-one','ca-one-vault-1','ca-one-vault',1,4294967295,1485469746);
INSERT INTO `aliases` (`identifier`,`pki_realm`,`alias`,`group_id`,`generation`,`notafter`,`notbefore`) VALUES ('iBAq6FOll0nqgIvHx3ocq2TZg94','ca-one','root-1','root',1,4294967295,1485469746);

INSERT INTO `certificate` (`pki_realm`,`issuer_dn`,`cert_key`,`issuer_identifier`,`identifier`,`subject`,`status`,`subject_key_identifier`,`authority_key_identifier`,`notbefore`,`notafter`,`req_key`,`public_key`,`data`) VALUES ('ca-one','CN=Root CA,OU=Test CA,DC=OpenXPKI,DC=ORG',1,'iBAq6FOll0nqgIvHx3ocq2TZg94','mIqp8XGxjquD0MtxsiFhv7JFfGk','CN=CA ONE,OU=Test CA,DC=OpenXPKI,DC=ORG','ISSUED','C0:2B:40:43:26:D3:98:8C:4D:2A:30:32:5F:30:D8:24:AA:26:4E:2D','43:EF:7A:11:FC:96:8B:35:6D:56:CC:17:C2:63:68:73:3C:B3:57:8D',1485469746,4294967295,NULL,'Public-Key: (2048 bit)\nModulus:\n    00:e4:d0:01:f7:46:ce:3b:67:f3:7d:1b:d1:41:cf:\n    f9:1e:ee:f5:b7:96:a1:79:27:d6:22:a4:d4:e7:a1:\n    29:56:ba:96:ec:12:8a:f0:78:19:b3:59:9f:61:df:\n    c5:fa:f2:26:3b:75:6f:b3:4b:33:c7:d1:28:42:c0:\n    cb:c1:78:ad:cd:44:7b:a8:92:8c:da:ab:7c:06:e8:\n    2f:45:2d:4b:46:80:8a:92:b0:9a:db:64:b0:98:b5:\n    ce:2a:90:fb:b1:96:62:28:52:c2:0b:cf:b5:a9:9d:\n    0f:50:1a:07:1c:60:29:55:b9:b1:dd:3e:b8:2c:27:\n    5d:c6:c5:04:68:0f:0c:12:6c:82:77:c5:4a:77:a2:\n    38:c1:0c:21:4b:9a:12:a3:9d:7b:57:5f:24:f8:e2:\n    c0:dc:10:57:be:69:e3:44:c8:99:e2:a0:a4:c4:c6:\n    e8:60:5f:e9:fc:b5:b2:2d:df:b4:08:5d:14:ae:92:\n    0e:a3:c0:79:96:a7:70:b4:f4:e7:41:b6:e2:db:d4:\n    c9:6c:68:e3:4b:3a:38:7b:b8:10:95:ee:dc:69:d3:\n    f7:28:58:d3:f4:4c:8c:3f:c8:7f:9d:79:f7:9a:38:\n    f8:7d:54:04:4b:3f:08:f0:ee:03:57:44:c2:90:9b:\n    66:0d:7d:8e:39:9e:c5:a5:51:46:55:4d:05:ed:de:\n    8f:47\nExponent: 65537 (0x10001)\n','-----BEGIN CERTIFICATE-----\nMIIDgjCCAmqgAwIBAgIBATANBgkqhkiG9w0BAQsFADBTMRMwEQYKCZImiZPyLGQB\nGRYDT1JHMRgwFgYKCZImiZPyLGQBGRYIT3BlblhQS0kxEDAOBgNVBAsMB1Rlc3Qg\nQ0ExEDAOBgNVBAMMB1Jvb3QgQ0EwIBcNMTcwMTI2MjIyOTA2WhgPMjExNzAxMDIy\nMjI5MDZaMFIxEzARBgoJkiaJk/IsZAEZFgNPUkcxGDAWBgoJkiaJk/IsZAEZFghP\ncGVuWFBLSTEQMA4GA1UECwwHVGVzdCBDQTEPMA0GA1UEAwwGQ0EgT05FMIIBIjAN\nBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA5NAB90bOO2fzfRvRQc/5Hu71t5ah\neSfWIqTU56EpVrqW7BKK8HgZs1mfYd/F+vImO3Vvs0szx9EoQsDLwXitzUR7qJKM\n2qt8BugvRS1LRoCKkrCa22SwmLXOKpD7sZZiKFLCC8+1qZ0PUBoHHGApVbmx3T64\nLCddxsUEaA8MEmyCd8VKd6I4wQwhS5oSo517V18k+OLA3BBXvmnjRMiZ4qCkxMbo\nYF/p/LWyLd+0CF0UrpIOo8B5lqdwtPTnQbbi29TJbGjjSzo4e7gQle7cadP3KFjT\n9EyMP8h/nXn3mjj4fVQESz8I8O4DV0TCkJtmDX2OOZ7FpVFGVU0F7d6PRwIDAQAB\no2AwXjAdBgNVHQ4EFgQUwCtAQybTmIxNKjAyXzDYJKomTi0wHwYDVR0jBBgwFoAU\nQ+96EfyWizVtVswXwmNoczyzV40wDwYDVR0TAQH/BAUwAwEB/zALBgNVHQ8EBAMC\nAQYwDQYJKoZIhvcNAQELBQADggEBALJWwfZGpOSSaYCh0yhBzLFntl+YCLnRb3L7\nAbVJTV1iB+8aP4k1N7GY/EO9cEBxgHb+hcX3pxpszwqCiGRzxWnOhhHeDkg7Q2zP\nfAJ6DlN+A9w29ffHjAB88yLmTth55X0Ek5ix7lPIlOPNmBTQ9aXjCJPG7XtEl+xz\ncIb7LUwUjlfUaIwf8WZb33e5C5GIptYUMZIktWkmI+rZ9aym9F4hy5Bx1tSHRfJV\npzTZ/NP4qhEi5eAXpj3LIC+UAdpflN4pTTMtTyE9RTR0k4nHQDjGeuWBrzZX3pc5\nXFW6Jm044cydqEzTKYDTEjH6zvhlfZmIHBbcpr34ZwldiXjbFE4=\n-----END CERTIFICATE-----');
INSERT INTO `certificate` (`pki_realm`,`issuer_dn`,`cert_key`,`issuer_identifier`,`identifier`,`subject`,`status`,`subject_key_identifier`,`authority_key_identifier`,`notbefore`,`notafter`,`req_key`,`public_key`,`data`) VALUES ('ca-one','CN=Root CA,OU=Test CA,DC=OpenXPKI,DC=ORG',2,'iBAq6FOll0nqgIvHx3ocq2TZg94','NZA-k3-yaMBQdsiAkbcfAeo8vQo','CN=SCEP,OU=Test CA,DC=OpenXPKI,DC=ORG','ISSUED','48:7F:FC:68:9A:B9:16:12:3A:60:C1:BA:D1:6A:EA:0D:24:8A:98:09','43:EF:7A:11:FC:96:8B:35:6D:56:CC:17:C2:63:68:73:3C:B3:57:8D',1485469746,4294967295,NULL,'Public-Key: (2048 bit)\nModulus:\n    00:a2:77:da:c9:71:09:74:63:45:a7:6d:c0:6d:32:\n    d5:80:39:d0:17:4a:bd:32:21:f1:94:94:7f:ab:42:\n    1a:73:fa:fe:fd:10:64:09:85:51:7f:7c:7a:60:24:\n    9e:40:2e:01:8f:bc:e3:08:e6:fc:95:05:57:88:d1:\n    6e:bf:5c:9e:df:02:38:a3:21:13:d6:d8:ea:c2:f7:\n    e9:0c:a5:ba:5f:7b:99:c7:f1:b8:3f:fb:ad:2c:c0:\n    6a:d3:a2:7a:87:65:b5:e1:89:29:bd:ed:30:64:5b:\n    40:f7:b7:58:61:fb:10:1d:bd:34:0f:8d:9c:a5:b0:\n    e6:b6:b8:bc:ff:06:96:1a:80:62:2f:6c:29:d9:de:\n    c3:9c:fb:a8:a3:51:7f:59:53:bb:59:08:fa:fd:1a:\n    af:23:10:39:bc:ce:fa:fd:93:ac:ad:64:1c:0a:d3:\n    83:9d:ea:c7:a7:2d:08:26:4e:b6:0a:cd:72:11:8a:\n    45:d0:78:bd:3f:8d:e8:a0:f7:1b:bb:5a:50:84:89:\n    cc:a4:d2:a3:3b:42:77:46:be:51:1b:fc:d0:e3:b2:\n    b9:c7:ef:57:65:46:2b:c5:b2:ce:e9:f6:ad:de:e8:\n    d8:9a:89:b8:67:da:1c:0e:cc:ab:58:8c:72:ad:16:\n    ea:67:7b:ce:c2:ed:e1:9c:1c:17:a9:d4:aa:1b:58:\n    30:ff\nExponent: 65537 (0x10001)\n','-----BEGIN CERTIFICATE-----\nMIIDbTCCAlWgAwIBAgIBAjANBgkqhkiG9w0BAQsFADBTMRMwEQYKCZImiZPyLGQB\nGRYDT1JHMRgwFgYKCZImiZPyLGQBGRYIT3BlblhQS0kxEDAOBgNVBAsMB1Rlc3Qg\nQ0ExEDAOBgNVBAMMB1Jvb3QgQ0EwIBcNMTcwMTI2MjIyOTA2WhgPMjExNzAxMDIy\nMjI5MDZaMFAxEzARBgoJkiaJk/IsZAEZFgNPUkcxGDAWBgoJkiaJk/IsZAEZFghP\ncGVuWFBLSTEQMA4GA1UECwwHVGVzdCBDQTENMAsGA1UEAwwEU0NFUDCCASIwDQYJ\nKoZIhvcNAQEBBQADggEPADCCAQoCggEBAKJ32slxCXRjRadtwG0y1YA50BdKvTIh\n8ZSUf6tCGnP6/v0QZAmFUX98emAknkAuAY+84wjm/JUFV4jRbr9cnt8COKMhE9bY\n6sL36Qylul97mcfxuD/7rSzAatOieodlteGJKb3tMGRbQPe3WGH7EB29NA+NnKWw\n5ra4vP8GlhqAYi9sKdnew5z7qKNRf1lTu1kI+v0aryMQObzO+v2TrK1kHArTg53q\nx6ctCCZOtgrNchGKRdB4vT+N6KD3G7taUISJzKTSoztCd0a+URv80OOyucfvV2VG\nK8Wyzun2rd7o2JqJuGfaHA7Mq1iMcq0W6md7zsLt4ZwcF6nUqhtYMP8CAwEAAaNN\nMEswCQYDVR0TBAIwADAdBgNVHQ4EFgQUSH/8aJq5FhI6YMG60WrqDSSKmAkwHwYD\nVR0jBBgwFoAUQ+96EfyWizVtVswXwmNoczyzV40wDQYJKoZIhvcNAQELBQADggEB\nAL78vaGME0wKR0PY4uqglWvTWyOzd/AqjvQGd1Y7TlcywXSYtMbRDiA4Ix4M4NLX\n/iAgjihadbQv0WOc/lvOfFplS+THnioMoz5U121//jiQJ8XB+sb8lDsz/iVevQ0/\nSm/xspfsML317Q+/C1Uo7TMCebw4QgZRVscGsSgZo4gQyriwCCdF2lt4jYB92Yu6\niymGmWm7nHrR+J7WVbMxd0NZicycOz6QOOGYXqAH61Fqty8h5Dc/7p2oKRaRHDj4\nPtKd5KbVlgVATIuizUsp34eIHwu+sXa4yoNM4RcYvz/OQB8C0Kv9XFTadEvBPGf6\nObbXnR37SfSv8ezX26Tscrc=\n-----END CERTIFICATE-----');
INSERT INTO `certificate` (`pki_realm`,`issuer_dn`,`cert_key`,`issuer_identifier`,`identifier`,`subject`,`status`,`subject_key_identifier`,`authority_key_identifier`,`notbefore`,`notafter`,`req_key`,`public_key`,`data`) VALUES (NULL,'CN=Root CA,OU=Test CA,DC=OpenXPKI,DC=ORG',16184103166370840861,'iBAq6FOll0nqgIvHx3ocq2TZg94','iBAq6FOll0nqgIvHx3ocq2TZg94','CN=Root CA,OU=Test CA,DC=OpenXPKI,DC=ORG','ISSUED','43:EF:7A:11:FC:96:8B:35:6D:56:CC:17:C2:63:68:73:3C:B3:57:8D','43:EF:7A:11:FC:96:8B:35:6D:56:CC:17:C2:63:68:73:3C:B3:57:8D',1485469746,4294967295,NULL,'Public-Key: (2048 bit)\nModulus:\n    00:c8:95:7a:7c:8a:0c:49:c2:bd:71:e1:1a:d8:93:\n    81:a8:17:30:b3:80:f7:0d:c1:e9:7a:49:77:f4:6f:\n    ce:9f:73:c1:f2:65:86:13:d4:4d:e9:56:84:1e:21:\n    bc:5c:99:bc:db:9f:12:69:59:8f:0b:47:29:35:a0:\n    8d:85:50:da:86:7c:a6:28:8d:5f:41:26:30:eb:f2:\n    70:fd:5d:ae:a5:06:48:bd:36:7a:0d:af:0a:2a:b6:\n    54:76:b5:d6:30:c4:d1:d0:6a:97:fb:34:02:21:54:\n    8e:dd:b0:20:89:cb:65:22:74:b7:f4:cd:b1:2e:84:\n    5d:19:1b:dd:cc:91:da:25:b9:36:3e:45:1e:65:85:\n    9d:cd:02:c6:fe:74:8c:98:3f:bc:93:0e:d5:5e:f8:\n    2a:64:39:46:5e:29:39:88:35:aa:30:9a:bd:3f:70:\n    55:71:54:19:51:4a:10:42:f7:c5:3e:36:0a:e9:1e:\n    09:9b:d1:cc:85:e2:54:c5:87:3d:65:d0:bc:51:2e:\n    4a:cb:d3:af:b5:e8:55:53:8c:99:4a:3c:52:0d:06:\n    c3:e5:b4:5a:56:eb:4e:b7:71:38:68:55:f1:2c:89:\n    7f:69:a2:99:e0:03:ff:fe:a9:db:5e:96:38:e1:9f:\n    69:53:17:90:95:d6:96:f7:74:e5:d5:bc:0d:dd:7b:\n    63:71\nExponent: 65537 (0x10001)\n','-----BEGIN CERTIFICATE-----\nMIIDizCCAnOgAwIBAgIJAOCZfBUvzwEdMA0GCSqGSIb3DQEBCwUAMFMxEzARBgoJ\nkiaJk/IsZAEZFgNPUkcxGDAWBgoJkiaJk/IsZAEZFghPcGVuWFBLSTEQMA4GA1UE\nCwwHVGVzdCBDQTEQMA4GA1UEAwwHUm9vdCBDQTAgFw0xNzAxMjYyMjI5MDZaGA8y\nMTE3MDEwMjIyMjkwNlowUzETMBEGCgmSJomT8ixkARkWA09SRzEYMBYGCgmSJomT\n8ixkARkWCE9wZW5YUEtJMRAwDgYDVQQLDAdUZXN0IENBMRAwDgYDVQQDDAdSb290\nIENBMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAyJV6fIoMScK9ceEa\n2JOBqBcws4D3DcHpekl39G/On3PB8mWGE9RN6VaEHiG8XJm8258SaVmPC0cpNaCN\nhVDahnymKI1fQSYw6/Jw/V2upQZIvTZ6Da8KKrZUdrXWMMTR0GqX+zQCIVSO3bAg\nictlInS39M2xLoRdGRvdzJHaJbk2PkUeZYWdzQLG/nSMmD+8kw7VXvgqZDlGXik5\niDWqMJq9P3BVcVQZUUoQQvfFPjYK6R4Jm9HMheJUxYc9ZdC8US5Ky9OvtehVU4yZ\nSjxSDQbD5bRaVutOt3E4aFXxLIl/aaKZ4AP//qnbXpY44Z9pUxeQldaW93Tl1bwN\n3XtjcQIDAQABo2AwXjAdBgNVHQ4EFgQUQ+96EfyWizVtVswXwmNoczyzV40wHwYD\nVR0jBBgwFoAUQ+96EfyWizVtVswXwmNoczyzV40wDwYDVR0TAQH/BAUwAwEB/zAL\nBgNVHQ8EBAMCAQYwDQYJKoZIhvcNAQELBQADggEBAKLV/G0HGSxEU0V3Q1dsEQ9V\niZYMpB/fpL0wo3+b/ZEiMXRX2xPvJvwT1RMslNbYg+BYxwXm0kLeRRwg73ZXpP0h\niXocRonKah7KJrQ5vtb8ZGsbULHdnbbYdOIHJU/Je4U3B/lKHrh9PzSb4sXIg1Ld\nPYeO4sGZd8v1q70L+KFr1JQ8qPDjKJbaylIws4mm6h5AzjOa16lVw05xj9aabDmS\nazyFn67Jlp7z4kn0cNQ5LZll2vzQGTAcDaTqa7WVScm00UxbFa7KEc2op9HE2Bfb\nIhtMOnP4FXjvbK30oJ6D5EOhIqJmgrZFKLgD9CaV6iWTvnAfRrmP44r3qmnEO+E=\n-----END CERTIFICATE-----');
INSERT INTO `certificate` (`pki_realm`,`issuer_dn`,`cert_key`,`issuer_identifier`,`identifier`,`subject`,`status`,`subject_key_identifier`,`authority_key_identifier`,`notbefore`,`notafter`,`req_key`,`public_key`,`data`) VALUES ('ca-one','CN=DataVault,DC=OpenXPKI Internal',10782130252911634402,'IPkm4dSkDn1oPuWIRZjIxvvpsno','IPkm4dSkDn1oPuWIRZjIxvvpsno','CN=DataVault,DC=OpenXPKI Internal','ISSUED','20:1A:DE:5E:52:3F:AC:CD:91:97:0A:C6:D2:5A:44:C6:74:A0:B9:53','20:1A:DE:5E:52:3F:AC:CD:91:97:0A:C6:D2:5A:44:C6:74:A0:B9:53',1485469746,4294967295,NULL,'Public-Key: (2048 bit)\nModulus:\n    00:d3:e9:e4:3f:4d:0e:ac:48:44:5b:bc:e9:f0:22:\n    09:ad:ee:d5:20:c6:60:aa:97:27:7a:e5:ec:ab:73:\n    19:82:72:81:bf:ce:ea:d9:29:ad:68:85:6e:16:0f:\n    f6:0d:e5:7a:90:68:25:03:72:f1:ef:c1:b8:63:79:\n    6c:88:ee:14:74:be:16:05:4b:c9:4c:8e:c0:44:42:\n    98:bf:96:45:2d:c4:0d:65:86:83:ea:13:7b:43:3a:\n    e3:f4:70:70:d5:5c:e5:76:f6:d8:54:33:b1:26:40:\n    53:1f:ad:b7:a9:e9:55:52:6d:2c:bf:ac:26:0b:1e:\n    5e:48:4e:4f:2a:65:40:e7:a6:0c:3b:be:aa:aa:60:\n    64:02:b6:92:1b:17:7d:14:74:4d:bb:a9:ac:c9:70:\n    7f:b2:87:04:94:c9:91:67:77:c4:91:17:24:ed:bb:\n    10:32:aa:e2:2a:32:81:1d:3c:8c:89:de:83:54:f9:\n    af:15:44:35:ab:44:b6:6a:c4:38:b9:5c:02:6e:e5:\n    59:5c:7d:cd:5a:d4:b6:68:37:61:64:3a:65:ef:0d:\n    0b:62:7d:c9:bb:a2:3a:25:5c:a9:03:31:dd:b7:55:\n    ff:7d:5a:ce:59:f0:96:6f:5e:ff:07:f4:15:88:e7:\n    99:39:83:91:74:3e:6f:00:b9:8c:cb:ac:62:59:0e:\n    8a:ff\nExponent: 65537 (0x10001)\n','-----BEGIN CERTIFICATE-----\nMIIDYjCCAkqgAwIBAgIJAJWh0la+ymfiMA0GCSqGSIb3DQEBCwUAMDcxITAfBgoJ\nkiaJk/IsZAEZFhFPcGVuWFBLSSBJbnRlcm5hbDESMBAGA1UEAwwJRGF0YVZhdWx0\nMCAXDTE3MDEyNjIyMjkwNloYDzIxMTcwMTAyMjIyOTA2WjA3MSEwHwYKCZImiZPy\nLGQBGRYRT3BlblhQS0kgSW50ZXJuYWwxEjAQBgNVBAMMCURhdGFWYXVsdDCCASIw\nDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBANPp5D9NDqxIRFu86fAiCa3u1SDG\nYKqXJ3rl7KtzGYJygb/O6tkprWiFbhYP9g3lepBoJQNy8e/BuGN5bIjuFHS+FgVL\nyUyOwERCmL+WRS3EDWWGg+oTe0M64/RwcNVc5Xb22FQzsSZAUx+tt6npVVJtLL+s\nJgseXkhOTyplQOemDDu+qqpgZAK2khsXfRR0TbuprMlwf7KHBJTJkWd3xJEXJO27\nEDKq4ioygR08jIneg1T5rxVENatEtmrEOLlcAm7lWVx9zVrUtmg3YWQ6Ze8NC2J9\nybuiOiVcqQMx3bdV/31azlnwlm9e/wf0FYjnmTmDkXQ+bwC5jMusYlkOiv8CAwEA\nAaNvMG0wCQYDVR0TBAIwADAdBgNVHQ4EFgQUIBreXlI/rM2RlwrG0lpExnSguVMw\nHwYDVR0jBBgwFoAUIBreXlI/rM2RlwrG0lpExnSguVMwCwYDVR0PBAQDAgUgMBMG\nA1UdJQQMMAoGCCsGAQUFBwMEMA0GCSqGSIb3DQEBCwUAA4IBAQBZvsVESue/IJa6\nE5+dfFO3keVUi9nB+1riyo/N3ZKCjhXev0vvNYdFyJWWJZUn+YpHVkcGt4H0B9tJ\nnEfQi2nWrLryQVEbBZg36SGODHyxCnOWzAyy+aYr6TS8W0Cf1GGYfhvToEy1A+QX\nbQZqsh0fD8+k+nmm1xGhMUQf5tC8tboEFvpN+4lyxjNlp0d8l9bElVEotjQreJrp\nf1rzzBZOx5cARYZa0iEFa5Ciwok73FMi+Y5JNjBPtwQVfunw6ECI8qO5Eg681GqK\nSCtHGFMene9rwFFj0wpQ7oogQLfQqQSDKkxcYRlOr2QDZQ5C8zicmSLNrJMQnJNp\nRULPx/So\n-----END CERTIFICATE-----');

__SQL
